# SPDX-License-Identifier: CC0-1.0
import struct


def pixel_de4_to_rai4(i, fn):
    with open(fn, 'wb+') as o:
        data_big = i.read(4)
        times = 0
        low = b''
        high = b''
        while data_big:
            pixels, = struct.unpack('<I', data_big)
            rom_pixels = ((pixels & 0x0000_0001) << 7  # -15 + 8
                          | (pixels & 0x0000_0002) << 2  # -10 + 8
                          | (pixels & 0x0000_0004) << 13  # -5 - 8
                          | (pixels & 0x0000_0008) << 8  # 0 - 8

                          | (pixels & 0x0000_0100) >> 2  # -6 + 8
                          | (pixels & 0x0000_0200) >> 7  # -1 + 8
                          | (pixels & 0x0000_0400) << 4  # 4 - 8
                          | (pixels & 0x0000_0800) >> 1  # 9 - 8

                          | (pixels & 0x0001_0000) >> 11  # 3 + 8
                          | (pixels & 0x0002_0000) >> 16  # 8 + 8
                          | (pixels & 0x0004_0000) >> 5  # 13 - 8
                          | (pixels & 0x0008_0000) >> 10  # 18 - 8

                          | (pixels & 0x0100_0000) >> 20  # 12 + 8
                          | (pixels & 0x0200_0000) >> 25  # 17 + 8
                          | (pixels & 0x0400_0000) >> 14  # 22 - 8
                          | (pixels & 0x0800_0000) >> 19)  # 27 - 8

            assert not (rom_pixels & ~0xFFFF)
            packed = struct.pack('<H', rom_pixels)
            if (times % 4) < 2:
                low += packed
            else:
                high += packed

            data_big = i.read(4)
            times += 1
            if times == 64:
                o.write(low)
                o.write(high)
                low = b''
                high = b''
                times = 0
            assert times < 64


def pixel_de5_to_spi6(inp, fn, fix0=False, fix1=False):
    with open(fn, 'wb+') as out:
        group = 0
        data = inp.read(4)
        while data:
            pixels, = struct.unpack('<I', data)

            # Pixel color fixes
            for i in range(4):
                n = i*8
                if fix0 and pixels >> n & 0xFF == 0x1F:
                    pixels = pixels | 0x20 << n
            # Fix for rdft_gfxdata1.bin
            if fix1 and 61_456 <= group and group <= 61_487:
                pixels = pixels | 0x2020_2020

            # Change format
            spi6_pixels2 = ((pixels & 0x0000_0001) << 16  # 0 + 16 + 0
                            | (pixels & 0x0000_0002) << 19  # -1 + 20 + 0
                            | (pixels & 0x0000_0004) << 6  # -2 + 8 + 0
                            | (pixels & 0x0000_0008) << 9  # -3 + 12 + 0
                            | (pixels & 0x0000_0010) >> 4  # -4 + 0 + 0
                            | (pixels & 0x0000_0020) >> 1  # -5 + 4 + 0

                            | (pixels & 0x0000_0100) << 9  # -8 + 16 + 1
                            | (pixels & 0x0000_0200) << 12  # -9 + 20 + 1
                            | (pixels & 0x0000_0400) >> 1  # -10 + 8 + 1
                            | (pixels & 0x0000_0800) << 2  # -11 + 12 + 1
                            | (pixels & 0x0000_1000) >> 11  # -12 + 0 + 1
                            | (pixels & 0x0000_2000) >> 8  # -13 + 4 + 1

                            | (pixels & 0x0001_0000) << 2  # -16 + 16 + 2
                            | (pixels & 0x0002_0000) << 5  # -17 + 20 + 2
                            | (pixels & 0x0004_0000) >> 8  # -18 + 8 + 2
                            | (pixels & 0x0008_0000) >> 5  # -19 + 12 + 2
                            | (pixels & 0x0010_0000) >> 18  # -20 + 0 + 2
                            | (pixels & 0x0020_0000) >> 15  # -21 + 4 + 2

                            | (pixels & 0x0100_0000) >> 5  # -24 + 16 + 3
                            | (pixels & 0x0200_0000) >> 2  # -25 + 20 + 3
                            | (pixels & 0x0400_0000) >> 15  # -26 + 8 + 3
                            | (pixels & 0x0800_0000) >> 12  # -27 + 12 + 3
                            | (pixels & 0x1000_0000) >> 25  # -28 + 0 + 3
                            | (pixels & 0x2000_0000) >> 22)  # -29 + 4 + 3

            assert not (spi6_pixels2 & ~0xFF_FFFF)
            packed32 = struct.pack('<I', spi6_pixels2)
            packed = packed32[:-1]
            out.write(packed)

            group += 1
            data = inp.read(4)


def pixel_de4_to_dd4_tile8(i, fn):
    with open(fn, 'wb+') as o:
        data_big = i.read(2)
        times = 0
        packed_tile = bytearray(32)
        while data_big:
            pixels, = struct.unpack('<H', data_big)
            rom_pixels = ((pixels & 0x0001)
                          | (pixels & 0x0002) << 1
                          | (pixels & 0x0004) << 2
                          | (pixels & 0x0008) << 3

                          | (pixels & 0x0100) >> 7
                          | (pixels & 0x0200) >> 6
                          | (pixels & 0x0400) >> 5
                          | (pixels & 0x0800) >> 4)

            assert not (rom_pixels & ~0xFF)
            pos = (times%4)*8 + times//4
            packed_tile[pos] = rom_pixels

            data_big = i.read(2)
            times += 1
            if times == 32:
                times = 0
                o.write(packed_tile)
            assert times < 32


def pixel_de4_to_dd4_tile16(i, path_low, path_high):
    '''Convert from a DotEmu format to a "Double Dragon" 16x16 format

    The de4 (DotEmu) format holds 8 bits per pixel of which only the
    lowest 4 store data. Each byte read represents a pixel, the tiles
    have all their pixels together and ordered from left to right, top
    to bottom.

    de4_tile16 EXPLAIN

    path_low and path_high need to be Path like objects (with an open
    method)

    '''
    with path_low.open('wb+') as ol, path_high.open('wb+') as oh:
        data_big = i.read(4)
        times = 0
        packed_tile = bytearray(128)
        while data_big:
            pixels, = struct.unpack('<I', data_big)
            rom_pixels_low = ((pixels & 0x0001)
                              | (pixels & 0x0000_0002) << 3

                              | (pixels & 0x0000_0100) >> 7
                              | (pixels & 0x0000_0200) >> 4

                              | (pixels & 0x0001_0000) >> 14
                              | (pixels & 0x0002_0000) >> 11

                              | (pixels & 0x0100_0000) >> 21
                              | (pixels & 0x0200_0000) >> 18)

            rom_pixels_high = ((pixels & 0x0004) >> 2
                               | (pixels & 0x0000_0008) << 1

                               | (pixels & 0x0000_0400) >> 9
                               | (pixels & 0x0000_0800) >> 6

                               | (pixels & 0x0004_0000) >> 16
                               | (pixels & 0x0008_0000) >> 13

                               | (pixels & 0x0400_0000) >> 23
                               | (pixels & 0x0800_0000) >> 20)

            assert not (rom_pixels_low & ~0xFF)
            assert not (rom_pixels_high & ~0xFF)
            pos_low = (times % 4)*16 + times//4
            offset = 64
            packed_tile[pos_low] = rom_pixels_low
            packed_tile[offset + pos_low] = rom_pixels_high

            data_big = i.read(4)
            times += 1
            if times == 64:
                times = 0
                ol.write(packed_tile[:64])
                oh.write(packed_tile[64:])
            assert times < 64



def _cm_ssr_packed_to_planar(packed):
    '''Converts subsprite row in Code Mystics packed pixel format to Neo Geo
    planar pixel format
    '''
    planar = ((packed & 0x0100_0000)
              | (packed & 0x1000_0000) >> 3
              | (packed & 0x0001_0000) << 10
              | (packed & 0x0010_0000) << 7

              | (packed & 0x0000_0100) << 20
              | (packed & 0x0000_1000) << 17
              | (packed & 0x0000_0001) << 30
              | (packed & 0x0000_0010) << 27


              | (packed & 0x0400_0000) >> 10
              | (packed & 0x4000_0000) >> 13
              | (packed & 0x0004_0000)
              | (packed & 0x0040_0000) >> 3

              | (packed & 0x0000_0400) << 10
              | (packed & 0x0000_4000) << 7
              | (packed & 0x0000_0004) << 20
              | (packed & 0x0000_0040) << 17


              | (packed & 0x0200_0000) >> 17
              | (packed & 0x2000_0000) >> 20
              | (packed & 0x0002_0000) >> 7
              | (packed & 0x0020_0000) >> 10

              | (packed & 0x0000_0200) << 3
              | (packed & 0x0000_2000)
              | (packed & 0x0000_0002) << 13
              | (packed & 0x0000_0020) << 10


              | (packed & 0x0800_0000) >> 27
              | (packed & 0x8000_0000) >> 30
              | (packed & 0x0008_0000) >> 17
              | (packed & 0x0080_0000) >> 20

              | (packed & 0x0000_0800) >> 7
              | (packed & 0x0000_8000) >> 10
              | (packed & 0x0000_0008) << 3
              | (packed & 0x0000_0080))
    return planar


def _cm_ss_packed_to_planar(packed_subsprite, shift=0):
    '''Converts a subsprite in Code Mystics packed pixel format to the
    Neo Geo planar pixel format
    '''
    planar = []
    for packed_row in packed_subsprite:
        #packed16 = (packed_row >> shift) & 0xFFFF_FFFF
        #print(f'0x{packed16:08x} 0b{packed16:032b}')
        planar.append(_cm_ssr_packed_to_planar(packed_row >> shift))
    return planar


def _cm_sprite_packed_to_planar(packed):
    '''Converts a sprite in Code Mystics packed pixel format to the
    Neo Geo planar pixel format
    '''
    planar = _cm_ss_packed_to_planar(packed[:8])
    planar.extend(_cm_ss_packed_to_planar(packed[8:]))
    planar.extend(_cm_ss_packed_to_planar(packed[:8], 32))
    planar.extend(_cm_ss_packed_to_planar(packed[8:], 32))
    return planar


def pixel_cm4_to_ng4(i, o):
    '''Converts a sprite in Code Mystics format to the Neo Geo format
    '''
    data = i.read(128)
    while data:
        assert len(data) == 128
        packed = struct.unpack('>QQQQQQQQQQQQQQQQ', data)

        planar = _cm_sprite_packed_to_planar(packed)

        # for planar_row in planar:
        #    planar_row16 = planar_row
        #    print(f'0x{planar_row16:08x} 0b{planar_row16:032b}')
        o.write(struct.pack('>32I', *planar))

        data = i.read(128)
