# SPDX-License-Identifier: CC0-1.0


def split(i, parts):
    '''Splits a file in parts

    The parts might be of different sizes. But they must be all
    continuous

    '''
    for name, size in parts:
        buf = i.read(size)
        assert len(buf) == size
        with open(name, 'wb+') as o:
            o.write(buf)


def split_ranges(i, parts):
    for name, ranges in parts:
        with open(name, 'wb+') as o:
            for start, end in ranges:
                i.seek(start)
                buf = i.read(end-start)
                assert i.tell() == end
                assert len(buf) == end-start
                o.write(buf)


def deinterleave16(i, parts, limit=-1):
    with open(parts[0], 'wb+') as o0, open(parts[1], 'wb+') as o1:
        count = 0
        word = i.read(2) if count != limit else None
        while word:
            o0.write(word[0:1])
            o1.write(word[1:2])
            count += 1
            word = i.read(2) if count != limit else None


def deinterleave32(i, parts):
    with open(parts[0], 'wb+') as o0, open(parts[1], 'wb+') as o1, \
         open(parts[2], 'wb+') as o2, open(parts[3], 'wb+') as o3:
        word = i.read(4)
        while word:
            o0.write(word[0:1])
            o1.write(word[1:2])
            o2.write(word[2:3])
            o3.write(word[3:4])
            word = i.read(4)


def mkempty(fn, size):
    with open(fn, 'wb+') as o:
        o.seek(size-1)
        o.write(b'\x00')
