#!/usr/bin/env python3
# SPDX-License-Identifier: CC0-1.0
from pathlib import Path
import shutil

from convert.tools import split
from convert.graphics import pixel_de4_to_dd4_tile8
from convert.graphics import pixel_de4_to_dd4_tile16
from convert.mapped import SequentialFiles


def ddragon(outdir):
    ddragon = outdir / 'ddragon'
    ddragon.mkdir(exist_ok=True)

    with open('ddragon_hd6309.bin', 'rb') as i:
        maincpu_parts = [
            (ddragon / '21j-1-5.26', 0x8000),
            (ddragon / '21j-2-3.25', 0x8000),
            (ddragon / '21j-3.24', 0x8000),
            (ddragon / '21j-4-1.23', 0x8000)
        ]
        split(i, maincpu_parts)
    sub_source = Path('ddragon_hd63701.bin')
    sub = Path(ddragon / '21jm-0.ic55')
    shutil.copyfile(sub_source, sub)
    soundcpu_source = Path('ddragon_m6809.bin')
    soundcpu = Path(ddragon / '21j-0-1')
    shutil.copyfile(soundcpu_source, soundcpu)
    with open('ddragon_gfxdata1.bin', 'rb') as i:
        pixel_de4_to_dd4_tile8(i, ddragon / '21j-5')
    with open('ddragon_gfxdata2.bin', 'rb') as i:
        size = 0x01_0000

        low0 = ddragon / '21j-a'
        low1 = ddragon / '21j-b'
        low2 = ddragon / '21j-c'
        low3 = ddragon / '21j-d'

        high0 = ddragon / '21j-e'
        high1 = ddragon / '21j-f'
        high2 = ddragon / '21j-g'
        high3 = ddragon / '21j-h'

        low = SequentialFiles([low0, low1, low2, low3], [size]*4)
        high = SequentialFiles([high0, high1, high2, high3], [size]*4)
        pixel_de4_to_dd4_tile16(i, low, high)
    with open('ddragon_gfxdata3.bin', 'rb') as i:
        size = 0x01_0000

        low0 = ddragon / '21j-8'
        low1 = ddragon / '21j-9'

        high0 = ddragon / '21j-i'
        high1 = ddragon / '21j-j'

        low = SequentialFiles([low0, low1], [size]*2)
        high = SequentialFiles([high0, high1], [size]*2)
        pixel_de4_to_dd4_tile16(i, low, high)
    with open('ddragon_adpcm.bin', 'rb') as i:
        adpcm_parts = [
            (ddragon / '21j-6', 0x01_0000),
            (ddragon / '21j-7', 0x01_0000)
        ]
        split(i, adpcm_parts)
    with open('proms.bin', 'rb') as i:
        proms_parts = [
            (ddragon / '21j-k-0', 0x0100),
            (ddragon / '21j-l-0', 0x0200)
        ]
        split(i, proms_parts)


def ddragon2(outdir):
    ddragon2 = outdir / 'ddragon2'
    ddragon2.mkdir(exist_ok=True)

    with open('ddragon2_hd6309.bin', 'rb') as i:
        maincpu_parts = [
            (ddragon2 / '26a9-04.bin', 0x8000),
            (ddragon2 / '26aa-03.bin', 0x8000),
            (ddragon2 / '26ab-0.bin', 0x8000),
            (ddragon2 / '26ac-0e.63', 0x8000)
        ]
        split(i, maincpu_parts)
    sub_source = Path('ddragon2_z80sub.bin')
    sub = Path(ddragon2 / '26ae-0.bin')
    shutil.copyfile(sub_source, sub)
    soundcpu_source = Path('ddragon2_z80sound.bin')
    soundcpu = Path(ddragon2 / '26ad-0.bin')
    shutil.copyfile(soundcpu_source, soundcpu)
    with open('ddragon2_gfxdata1.bin', 'rb') as i:
        pixel_de4_to_dd4_tile8(i, ddragon2 / '26a8-0e.19')
    with open('ddragon2_gfxdata2.bin', 'rb') as i:
        low0 = ddragon2 / '26j0-0.bin'
        low1 = ddragon2 / '26j1-0.bin'
        low2 = ddragon2 / '26af-0.bin'

        high0 = ddragon2 / '26j2-0.bin'
        high1 = ddragon2 / '26j3-0.bin'
        high2 = ddragon2 / '26a10-0.bin'

        low = SequentialFiles([low0, low1, low2],
                              [0x02_0000, 0x02_0000, 0x02_0000])
        high = SequentialFiles([high0, high1, high2],
                               [0x02_0000, 0x02_0000, 0x02_0000])
        pixel_de4_to_dd4_tile16(i, low, high)
    with open('ddragon2_gfxdata3.bin', 'rb') as i:
        low = Path(ddragon2 / '26j4-0.bin')
        high = Path(ddragon2 / '26j5-0.bin')
        pixel_de4_to_dd4_tile16(i, low, high)
    with open('ddragon2_oki.bin', 'rb') as i:
        proms_parts = [
            (ddragon2 / '26j6-0.bin', 0x02_0000),
            (ddragon2 / '26j7-0.bin', 0x02_0000)
        ]
        split(i, proms_parts)
    with open('proms.bin', 'rb') as i:
        proms_parts = [
            ('unk_prom', 0x0100),
            (ddragon2 / 'prom.16', 0x0200)
        ]
        split(i, proms_parts)


def dragon_convert():
    outdir = Path('mame')
    outdir.mkdir(exist_ok=True)

    ddragon(outdir)
    ddragon2(outdir)


if __name__ == '__main__':
    dragon_convert()
