#!/usr/bin/env python3
# SPDX-License-Identifier: CC0-1.0
from pathlib import Path
import shutil

from convert import config
from convert.tools import split
from convert.graphics import pixel_cm4_to_ng4
from convert.mapped import InterleavedFiles, SequentialFiles


def lastbld2(outdir):
    lastbld2 = outdir / 'lastbld2'
    lastbld2.mkdir(exist_ok=True)

    with open('p1.bin', 'rb') as i:
        maincpu_parts = [
            (lastbld2 / '243-pg1.p1', 0x10_0000),
            (lastbld2 / '243-pg2.sp2', 0x40_0000),
        ]
        split(i, maincpu_parts)

    sfix_source = Path('s1.bin')
    sfix = lastbld2 / '243-s1.s1'
    shutil.copyfile(sfix_source, sfix)

    with open('v1.bin', 'rb') as i:
        ymsnd_parts = [
            (lastbld2 / '243-v1.v1', 0x40_0000),
            (lastbld2 / '243-v2.v2', 0x40_0000),
            (lastbld2 / '243-v3.v3', 0x40_0000),
            (lastbld2 / '243-v4.v4', 0x40_0000),
        ]
        split(i, ymsnd_parts)

    bios_audio_source = Path('m1.bin')
    bios_audio = lastbld2 / '243-m1.m1'
    shutil.copyfile(bios_audio_source, bios_audio)

    with open('c1.bin', 'rb') as i:
        files = [
            SequentialFiles([lastbld2 / '243-c1.c1', lastbld2 / '243-c3.c3',
                             lastbld2 / '243-c5.c5'],
                            [0x80_0000, 0x80_0000, 0x80_0000], 'wb'),
            SequentialFiles([lastbld2 / '243-c2.c2', lastbld2 / '243-c4.c4',
                             lastbld2 / '243-c6.c6'],
                            [0x80_0000, 0x80_0000, 0x80_0000], 'wb')
        ]
        with InterleavedFiles(files, 1, 'wb') as o:
            pixel_cm4_to_ng4(i, o)


def neogeo(outdir):
    neogeo = outdir / 'neogeo'
    neogeo.mkdir(exist_ok=True)

    japan_j3_source = Path('sp_4j.bin')
    japan_j3 = neogeo / 'japan-j3.bin'
    shutil.copyfile(japan_j3_source, japan_j3)


def lastblade2_convert():
    outdir = Path('mame')
    outdir.mkdir(exist_ok=True)

    lastbld2(outdir)
    if config["partial"]:
        neogeo(outdir)


if __name__ == '__main__':
    lastblade2_convert()
